<?php
/**
 * ASM Cluster node API.
 * Samples of implementation of basic API for manage of node application.
 *
 * @author bgv <bgv.asm.cluster@gmail.com>
 * @link http://asm-search-engine.com/cluster/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://asm-search-engine.com/cluster/license/
 * @package ASM Cluster node API
 * @since 0.1
 */

require_once 'zmsg.php';

/**
 * Define and init ACN protocol node handlers names.
 */
defined('ACN_HANDLER_ADMIN') or define('ACN_HANDLER_ADMIN', 'Admin');
defined('ACN_HANDLER_ROUTER_SERVER_PROXY') or define('ACN_HANDLER_ROUTER_SERVER_PROXY', 'RouterServerProxy');
defined('ACN_HANDLER_DATA_SERVER_PROXY') or define('ACN_HANDLER_DATA_SERVER_PROXY', 'DataServerProxy');
defined('ACN_HANDLER_DATA_CLIENT_PROXY') or define('ACN_HANDLER_DATA_CLIENT_PROXY', 'DataClientProxy');
defined('ACN_HANDLER_DATA_PROCESSOR_DATA') or define('ACN_HANDLER_DATA_PROCESSOR_DATA', 'DataProcessorData');
defined('ACN_HANDLER_DATA_CLIENT_DATA') or define('ACN_HANDLER_DATA_CLIENT_DATA', 'DataClientData');
defined('ACN_HANDLER_DATA_REDUCER_PROXY') or define('ACN_HANDLER_DATA_REDUCER_PROXY', 'DataReducerProxy');

/**
 * Define and init admin command names supported by all handlers.
 */
defined('ACN_CMD_ECHO') or define('ACN_CMD_ECHO', 'ECHO');
defined('ACN_CMD_LLGET') or define('ACN_CMD_LLGET', 'LLGET');
defined('ACN_CMD_LLSET') or define('ACN_CMD_LLSET', 'LLSET');

/**
 * Define and init admin command names supported by specific handlers.
 */
defined('ACN_CMD_STAT') or define('ACN_CMD_STAT', 'STAT');
defined('ACN_CMD_TIME') or define('ACN_CMD_TIME', 'TIME');
defined('ACN_CMD_PROPERTIES') or define('ACN_CMD_PROPERTIES', 'PROPERTIES');
defined('ACN_CMD_REBUILD_SERVER_CONNECTION') or define('ACN_CMD_REBUILD_SERVER_CONNECTION', 'REBUILD_SERVER_CONNECTION');
defined('ACN_CMD_DISCONNECT_SERVER_CONNECTION') or define('ACN_CMD_DISCONNECT_SERVER_CONNECTION', 'DISCONNECT_SERVER_CONNECTION');
defined('ACN_CMD_REBUILD_CLIENT_CONNECTION') or define('ACN_CMD_REBUILD_CLIENT_CONNECTION', 'REBUILD_CLIENT_CONNECTION');
defined('ACN_CMD_DISCONNECT_CLIENT_CONNECTION') or define('ACN_CMD_DISCONNECT_CLIENT_CONNECTION', 'DISCONNECT_CLIENT_CONNECTION');
defined('ACN_CMD_UPDATE_SCHEMA') or define('ACN_CMD_UPDATE_SCHEMA', 'UPDATE_SCHEMA');
defined('ACN_CMD_REBUILD_CLIENT_CONNECTION') or define('ACN_CMD_REBUILD_CLIENT_CONNECTION', 'REBUILD_CLIENT_CONNECTION');
defined('ACN_CMD_SHUTDOWN') or define('ACN_CMD_SHUTDOWN', 'SHUTDOWN');

/**
 * Define and init ACN protocol error codes.
 */
defined('ACN_PROTOCOL_ERROR_OK') or define('ACN_PROTOCOL_ERROR_OK', 0);
defined('ACN_PROTOCOL_ERROR_CONNECTION_PARAMS') or define('ACN_PROTOCOL_ERROR_CONNECTION_PARAMS', -1);
defined('ACN_PROTOCOL_ERROR_CONTEXT_CREATE') or define('ACN_PROTOCOL_ERROR_CONTEXT_CREATE', -2);
defined('ACN_PROTOCOL_ERROR_SOCKET_CREATE') or define('ACN_PROTOCOL_ERROR_SOCKET_CREATE', -3);
defined('ACN_PROTOCOL_ERROR_TIMEOUT') or define('ACN_PROTOCOL_ERROR_TIMEOUT', -5);


/**
 * Define and init ACN protocol node admin command names.
 */
defined('ACN_ADMIN_CMD_SPHINX') or define('ACN_ADMIN_CMD_SPHINX', 'SPHINX');
defined('ACN_ADMIN_CMD_ECHO') or define('ACN_ADMIN_CMD_ECHO', 'ECHO');
defined('ACN_ADMIN_CMD_STAT') or define('ACN_ADMIN_CMD_STAT', 'STAT');
defined('ACN_ADMIN_CMD_REBUILD_SERVER_CONNECTION') or define('ACN_ADMIN_CMD_REBUILD_SERVER_CONNECTION', 'REBUILD_SERVER_CONNECTION');
defined('ACN_ADMIN_CMD_DISCONNECT_SERVER_CONNECTION') or define('ACN_ADMIN_CMD_DISCONNECT_SERVER_CONNECTION', 'DISCONNECT_SERVER_CONNECTION');
defined('ACN_ADMIN_CMD_REBUILD_CLIENT_CONNECTION') or define('ACN_ADMIN_CMD_REBUILD_CLIENT_CONNECTION', 'REBUILD_CLIENT_CONNECTION');
defined('ACN_ADMIN_CMD_DISCONNECT_CLIENT_CONNECTION') or define('ACN_ADMIN_CMD_DISCONNECT_CLIENT_CONNECTION', 'DISCONNECT_CLIENT_CONNECTION');
defined('ACN_ADMIN_CMD_UPDATE_SCHEMA') or define('ACN_ADMIN_CMD_UPDATE_SCHEMA', 'UPDATE_SCHEMA');
defined('ACN_ADMIN_CMD_REBUILD_CLIENT_CONNECTION') or define('ACN_ADMIN_CMD_REBUILD_CLIENT_CONNECTION', 'REBUILD_CLIENT_CONNECTION');
defined('ACN_ADMIN_CMD_SHUTDOWN') or define('ACN_ADMIN_CMD_SHUTDOWN', 'SHUTDOWN');

/**
 * Define and init ACN protocol node admin state codes.
 */
defined('ACN_ADMIN_NODE_ADMIN_ERROR_OK') or define('ACN_ADMIN_NODE_ADMIN_ERROR_OK', 'OK');
defined('ACN_ADMIN_NODE_ADMIN_ERROR_ERROR') or define('ACN_ADMIN_NODE_ADMIN_ERROR_ERROR', 'ERROR');

/**
 * Define and init ACN protocol node admin command parts delimiter.
 */
defined('ACN_ADMIN_CMD_DELIMITER') or define('ACN_ADMIN_CMD_DELIMITER', "\t"/*':'*/);

/**
 * Define and init default admin connection protocol.
 */
defined('ACN_PROTOCOL_ADMIN_DEFAULT') or define('ACN_PROTOCOL_ADMIN_DEFAULT', 'tcp');
/**
 * Define and init default admin connection host.
 */
defined('ACN_HOST_ADMIN_DEFAULT') or define('ACN_HOST_ADMIN_DEFAULT', 'localhost');
/**
 * Define and init default admin connection port.
 */
defined('ACN_PORT_ADMIN_DEFAULT') or define('ACN_PORT_ADMIN_DEFAULT', 5548);
/**
 * Define and init default data connection port.
 */
defined('ACN_PORT_ROUTER_DEFAULT') or define('ACN_PORT_ROUTER_DEFAULT', 5557);
/**
 * Define and init default command delimiter.
 */
defined('ACN_CMD_DELIMITER') or define('ACN_CMD_DELIMITER', ':');
/**
 * Define and init client identity prefix.
 */
defined('ACN_CLIENT_IDENTITY_PREFIX') or define('ACN_CLIENT_IDENTITY_PREFIX', 'ASM-CLU-');
/**
 * Define and init message Id prefix.
 */
defined('ACN_MSG_ID_PREFIX') or define('ACN_MSG_ID_PREFIX', 'ID-');
/**
 * Define and init message requests` response timeout.
 */
defined('ACN_MSG_RESPONSE_TIMEOUT') or define('ACN_MSG_RESPONSE_TIMEOUT', 1500);
/**
 * Define and init connection types.
 */
defined('ACN_CONNECTION_TYPE_ADMIN') or define('ACN_CONNECTION_TYPE_ADMIN', 0);
defined('ACN_CONNECTION_TYPE_ROUTER') or define('ACN_CONNECTION_TYPE_ROUTER', 1);

/**
 * Define handler types for client request message.
 */
defined('ACN_HANDLER_TYPE_SPHINX') or define('ACN_HANDLER_TYPE_SPHINX', 0);
defined('ACN_HANDLER_TYPE_SQLITE') or define('ACN_HANDLER_TYPE_SQLITE', 1);
defined('ACN_HANDLER_TYPE_MYSQL') or define('ACN_HANDLER_TYPE_MYSQL', 2);


/*
 * @desc create ACN connection (new zmq context, zmq socket and connect using specified or default options)
 * @param $connection_array - array of connection configuration options: host, port, type {ACN_CONNECTION_TYPE_ADMIN, ACN_CONNECTION_TYPE_ROUTER}, identity - string of unique client Id
 *                            in cluster
 *
 * @return ACN connection array; error item is ACN_PROTOCOL_ERROR_OK - if success and negative error code: ACN_PROTOCOL_ERROR_CONNECTION_PARAMS - wrong one or several connection parameter,
 *                                             ACN_PROTOCOL_ERROR_CONTEXT_CREATE - error create zmq context, ACN_PROTOCOL_ERROR_SOCKET_CREATE - error create zmq socket
 */
 function acn_connection_create($connection_array=null){
   //Init returned connection
   $ret=array('error'=>ACN_PROTOCOL_ERROR_OK, 'context'=>null, 'socket'=>null, 'type'=>null, 'host'=>null, 'port'=>null, 'identity'=>null);

   //Check connection parameters
   if($connection_array===null){
     $con_ar=array('host'=>ACN_HOST_ADMIN_DEFAULT, 'port'=>ACN_PORT_ADMIN_DEFAULT, 'type'=>ACN_CONNECTION_TYPE_ADMIN, 'identity'=>acn_unique_client_id());
   }else{
     if(!isset($connection_array['host']) || !isset($connection_array['port']) || !isset($connection_array['type']) || !isset($connection_array['identity'])){
       $ret['error']=ACN_PROTOCOL_ERROR_CONNECTION_PARAMS;
     }else{
       $con_ar=$connection_array;
     }
   }

   //Create connection
   if($ret['error']==ACN_PROTOCOL_ERROR_OK){
     //Create zmq context
     $ret['context']=new ZMQContext();
     if($ret['context']==null){
       $ret['error']=ACN_PROTOCOL_ERROR_CONTEXT_CREATE;
     }else{
       //Create zmq socket
       $ret['socket']=new ZMQSocket($ret['context'], ZMQ::SOCKET_DEALER);
       if($ret['socket']==null){
         $ret['error']=ACN_PROTOCOL_ERROR_SOCKET_CREATE;
       }else{
         //Set request identity
         $ret['socket']->setSockOpt(ZMQ::SOCKOPT_IDENTITY, $con_ar['identity']);
         if($con_ar['type']==ACN_CONNECTION_TYPE_ADMIN){
           //Set linger 0, do not wait on close
           $ret['socket']->setSockOpt(ZMQ::SOCKOPT_LINGER, 0);
         }

         //Connect
         $ret['socket']->connect(ACN_PROTOCOL_ADMIN_DEFAULT.'://'.$con_ar['host'].':'.$con_ar['port']);

         //Set properties
         $ret['host']=$con_ar['host'];
         $ret['port']=$con_ar['port'];
         $ret['type']=$con_ar['type'];
         $ret['identity']=$con_ar['identity'];
       }
     }
   }

   return $ret;
 }

/*
 * @desc delete ACN connection (disconnect zmq socket, free zmq socket and context)
 * @param $connection_array - array of connection configuration options: host, port, type and identity
 *
 * @return ACN connection array in initial state
 */
 function acn_connection_delete(&$connection_array){
   $connection_array['context']=NULL;
   $connection_array['socket']=NULL;
   $connection_array['host']=NULL;
   $connection_array['port']=NULL;
   $connection_array['type']=NULL;
   $connection_array['identity']=NULL;

   return $connection_array;
 }

/*
 * @desc send ACN connection message
 * @param $acn_connection - acn connection returned by acn_connection_create() call
 *        $fields_array   - array of message fields 'id' and 'body'
 *
 * @return acn message array or error ACN_PROTOCOL_ERROR_FIELDS_NOT_SET - fields array not set proper way
 */
 function acn_message_send($acn_connection, $fields_array){
   //Init return acn message array
   $ret=array('error'=>ACN_PROTOCOL_ERROR_OK, 'message'=>null);

   if(!isset($fields_array['body']) || !isset($fields_array['id'])){
     $ret['error']=ACN_PROTOCOL_ERROR_FIELDS_NOT_SET;
   }else{
     //Create message
     $ret['message']=new Zmsg($acn_connection['socket']);

     //Set fields
     $ret['message']->wrap($fields_array['body'], NULL);
     $ret['message']->wrap($fields_array['id'], NULL);

     //Send message
     $ret['message']->send();
   }

   return $ret;
 }

/*
 * @desc receive ACN connection message
 * @param $acn_connection - acn connection returned by acn_connection_create() call
 *
 * @return ACN responses array or ACN_PROTOCOL_ERROR_TIMEOUT - if timeout reached
 */
 function acn_message_receive($acn_connection, $timeout=ACN_MSG_RESPONSE_TIMEOUT){
   //Init return acn response array
   $ret=array('error'=>ACN_PROTOCOL_ERROR_OK, 'messages'=>array());

   //Poll socket for a reply, with timeout
   $read=$write=array();
   $poll=new ZMQPoll();
   $poll->add($acn_connection['socket'], ZMQ::POLL_IN);
   $events=$poll->poll($read, $write, $timeout);
   //Handle events
   if($events){
     foreach($read as $socket) {
       $zmsg_r=new Zmsg($socket);
       $zmsg_r->recv();
       $ret['messages'][]=array('error'=>ACN_PROTOCOL_ERROR_OK, 'message'=>$zmsg_r, 'id'=>$zmsg_r->unwrap(), 'body'=>$zmsg_r->unwrap());
       //$zmsg_r->__toString()
     }
   }else{
     $ret['error']=ACN_PROTOCOL_ERROR_TIMEOUT;
   }
   $poll->remove($acn_connection['socket']);

   return $ret;
 }

/*
 * @desc generate unique client identifier
 *
 * @return unique client identifier string
 */
 function acn_unique_client_id($type=1, $prefix=null){
   if($prefix===null){
     $prefix=date('Y-m-d H:i:s').'-'.microtime(true).'-';
   }

   $ret=ACN_CLIENT_IDENTITY_PREFIX.$prefix;

   if($type==0){
     //Simple fast
     $ret.=acn_unique_id(1);
   }else{
     //More accurate complexity
     $ret.=acn_unique_id(3).'-'.acn_unique_id(5);
   }

   return $ret;
 }

/*
 * @desc generate unique message identifier
 *
 * @return unique message identifier string
 */
 function acn_unique_message_id($type=1, $prefix=''){
   $ret=ACN_MSG_ID_PREFIX.$prefix;

   if($type==0){
     //Simple fast
     $ret.=acn_unique_id(0);
   }else{
     //More accurate complexity
     //$ret.=acn_unique_id(3).'-'.acn_unique_id(5);
     $ret.=acn_unique_id(4);
   }

   return $ret;
 }

/*
 * @desc create ACN API admin message
 *
 * @return ACN API admin message string
 */
 function acn_admin_message_create($handler, $command, $parameters){
   return $handler.ACN_ADMIN_CMD_DELIMITER.$command.ACN_ADMIN_CMD_DELIMITER.$parameters.ACN_ADMIN_CMD_DELIMITER;
 }


/*
 * @desc generate unique identifier by one of several methods
 *
 * @return unique message identifier string
 */
 function acn_unique_id($type=0){
   $ret=null;

   if($type==0){
     //Complex string
     $ret=uniqid(md5(rand()), true);
   }elseif($type==1){
     //Complex string hexadec
     $ret=dechex(time()).dechex(mt_rand(1, 65535));
   }elseif($type==2){
     //Random for better crypto cypher hexadec
     $r=unpack('v*', fread(fopen('/dev/random', 'r'), 16));
     $ret=sprintf('%04x%04x-%04x-%04x-%04x-%04x%04x%04x', $r[1], $r[2], $r[3], $r[4] & 0x0fff | 0x4000, $r[5] & 0x3fff | 0x8000, $r[6], $r[7], $r[8]);
   }elseif($type==3){
     //Host name and microseconds
     $ret=uniqid(php_uname('n').'-', true);
   }elseif($type==4){
     //Another fast useful for file name
     $ret=time().substr(md5(microtime(true)), 0, rand(5, 12));
   }elseif($type==5){
     //The field names refer to RFC 4122 section 4.1.2
     $ret=sprintf('%04x%04x-%04x-%03x4-%04x-%04x%04x%04x',
                  mt_rand(0, 65535), mt_rand(0, 65535), // 32 bits for "time_low"
                  mt_rand(0, 65535), // 16 bits for "time_mid"
                  mt_rand(0, 4095),  // 12 bits before the 0100 of (version) 4 for "time_hi_and_version"
                  bindec(substr_replace(sprintf('%016b', mt_rand(0, 65535)), '01', 6, 2)),
                  // 8 bits, the last two of which (positions 6 and 7) are 01, for "clk_seq_hi_res"
                  // (hence, the 2nd hex digit after the 3rd hyphen can only be 1, 5, 9 or d)
                  // 8 bits for "clk_seq_low"
                  mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535) // 48 bits for "node"  
          );  
   }

   return $ret;
 }

?>