#!/usr/bin/php
<?php
/**
 * ASM Cluster key-value db manager API test client application.
 * Samples of implementation of basic client-side API for kv-db manager.
 *
 * @author bgv <bgv.asm.cluster@gmail.com>
 * @link http://asm-search-engine.com/cluster/
 * @copyright Copyright &copy; 2013 IOIX Ukraine
 * @license http://asm-search-engine.com/cluster/license/
 * @package ASM tools utilities API
 * @since 0.1
*/

 @ini_set('max_execution_time', 120);
 @ini_set('memory_limit', '1024M');

 @error_reporting(E_ALL);
 @ini_set('display_errors', 1);

 require_once 'acn_cli_api.inc.php';
 require_once 'acn_kvdb_api.inc.php';

 //Check cli runtime
 if(php_sapi_name()!=='cli' || !defined('STDIN')){
   echo 'Only cli execution mode supported'.PHP_EOL;
   exit(1);
 }

 //Parse cli arguments
 $args=cli_parse_arguments($argv);

 //Check mandatory arguments
 if(isset($args['h']) || isset($args['help']) || !isset($args[ACN_KVDB_CA_URL], $args['command'])){
   echo 'Usage: '.$argv[0].' --url=kvdb_manager_url> --command=<command{put,get,delete,check,name}> [--document=<document_file> --id=<document_id(s)>] '.
        '[--t=<http_requet_timeout_sec>] [--l=<log_mode{0-Off,1-On}>]'.PHP_EOL;

   exit(1);
 }

 //Set default values
 if(!isset($args[ACN_KVDB_CA_TIMEOUT]) || empty($args[ACN_KVDB_CA_TIMEOUT])){
   $args[ACN_KVDB_CA_TIMEOUT]=ACN_KVDB_TIMEOUT;
 }
 if(!isset($args['l']) || $args['l']===''){
   $args['l']=1;
 }

 //Prepare request array
 $aborted='';
 switch($args['command']){
   case 'put' : {
     if(!isset($args['document'])){
       $aborted='--document=<document_file> ';
     }else{
       if(!file_exists($args['document'])){
         $aborted.='file '.$args['document'].' ';
       }else{
         if(!isset($args['id'])){
           $aborted.='--id=<document_id>';
         }else{
           if(JSON_USE_BASE64){
             $documents=array($args['id']=>base64_encode(file_get_contents($args['document'])));
           }else{
             $documents=array($args['id']=>file_get_contents($args['document']));
           }
           $request_array=ACN_KVDB_prepare_request_array(ACN_KVDB_CMD_PUT, $documents);
         }
       }
     }
     break;
   }
   case 'check'  : {
     if(!isset($args['id'])){
       $aborted.='--id=<document_id>';
     }else{
       $documents=explode(',', $args['id']);
       $request_array=ACN_KVDB_prepare_request_array(ACN_KVDB_CMD_CHECK, $documents);
     }
     break;
   }
   case 'delete' : {
     if(!isset($args['id'])){
       $aborted.='--id=<document_id>';
     }else{
       $documents=explode(',', $args['id']);
       $request_array=ACN_KVDB_prepare_request_array(ACN_KVDB_CMD_DELETE, $documents);
     }
     break;
   }
   case 'get'    : {
     if(!isset($args['id'])){
       $aborted.='--id=<document_id>';
     }else{
       $documents=explode(',', $args['id']);
       $request_array=ACN_KVDB_prepare_request_array(ACN_KVDB_CMD_GET, $documents);
     }
     break;
   }
   case 'name'    : {
     $documents=array(ACN_KVDB_CMD_NAME);
     $request_array=ACN_KVDB_prepare_request_array(ACN_KVDB_CMD_MANAGE, $documents);
     break;
   }
 }

 //Execute request
 if($aborted==''){
   $r=ACN_KVDB_request($request_array, array(ACN_KVDB_CA_URL=>$args[ACN_KVDB_CA_URL], ACN_KVDB_CA_TIMEOUT=>$args[ACN_KVDB_CA_TIMEOUT]));
   if($r['error']==ACN_KVDB_ERROR_OK){
     $r=ACN_KVDB_parse_response($r['response']);
     if($args['l']){
       var_dump($r); echo PHP_EOL;
     }
     if($r[ACN_KVDB_FIELD_ERROR_CODE]==ACN_KVDB_ERROR_OK){
       echo cli_prettyPrintJson($r[ACN_KVDB_FIELD_DATA], '');
     }else{
       if($args['l']){
         echo 'Response error '.$r[ACN_KVDB_FIELD_ERROR_CODE].' : '.$r[ACN_KVDB_FIELD_ERROR_MSG];
       }
     }
   }else{
     $r[ACN_KVDB_FIELD_TIME]=0;
     if($args['l']){
       echo 'Request error '.$r['error'].' : '.$r['error_msg'].PHP_EOL;
     }
   }
 }else if($aborted!==''){
   $r[ACN_KVDB_FIELD_TIME]=0;
   if($args['l']){
     echo $aborted.' required'.PHP_EOL;
   }
 }

 //Out stat info
 if($args['l']){
   echo PHP_EOL.'Finished, time '.$r[ACN_KVDB_FIELD_TIME].PHP_EOL;
 }

 exit();

?>